package com.fr.plugins.dsgroup;

import com.fr.base.TableData;
import com.fr.base.TableDataBean;
import com.fr.config.RemoteConfigEvent;
import com.fr.design.data.datapane.TableDataPaneListPane;
import com.fr.esd.event.DSMapping;
import com.fr.esd.event.DsNameTarget;
import com.fr.esd.event.StrategyEventsNotifier;
import com.fr.event.EventDispatcher;
import com.fr.file.*;
import com.fr.general.NameObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugins.dsgroup.fun.DSGroupTableData;
import com.fr.plugins.dsgroup.rpc.GroupTableDataOperator;
import com.fr.stable.Nameable;
import com.fr.stable.StringUtils;
import com.fr.third.org.apache.commons.collections4.MapUtils;
import com.fr.workspace.WorkContext;


import javax.swing.*;
import java.util.*;

/**
 * Created by phetran_ye on 2019-03-12.
 * 服务器数据集修改弹窗
 */
public class GroupTableDataPaneListPane extends TableDataPaneListPane {

    /**
     * TableDataManagerPane下面有这个pane，
     * 这个managerPane初始化的时候会将pane的rename方法覆盖
     * 下面新增的部分就是覆盖逻辑
     * 参考 TableDataManagerPane的initTableDataManagerPane方法
     * @param oldName
     * @param newName
     */
    @Override
    public void rename(String oldName, String newName) {
        super.rename(oldName, newName);
        (new SwingWorker<Void, Void>() {
            @Override
            protected Void doInBackground() {
                StrategyEventsNotifier.modifyDataSet(new DSMapping[]{DSMapping.ofServerDS(new DsNameTarget(oldName))});
                return null;
            }
        }).execute();


    }


    @Override
    public void update(TableDataConfig dataConfig){
        Nameable[] tableDataArray = this.update();

        //增加逻辑
        this.handleUpdateRelatedWithGroup(tableDataArray);

        final List<TableDataBean> tableDatas = new ArrayList();
        Map<String,String> nameChangedMap = MapUtils.invertMap(this.getDsNameChangedMap());
        for(int i=0;i<tableDataArray.length;i++){
            NameObject nameObject = (NameObject)tableDataArray[i];
            String oldName=nameChangedMap.get(nameObject.getName());
            if(oldName==null){
                oldName= StringUtils.EMPTY;
            }
            tableDatas.add(new TableDataBean(nameObject.getName(), oldName, (TableData)nameObject.getObject()));
        }
        try {
            WorkContext.getCurrent().get(GroupTableDataOperator.class).saveTableData(new ArrayList(dataConfig.getTableDatas().keySet()),tableDatas);
            if (!WorkContext.getCurrent().isLocal()) {
                EventDispatcher.fire(RemoteConfigEvent.EDIT, TableDataConfig.getInstance().getNameSpace());
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }

    }

    /**
     * 处理涉及分组的服务器数据集的删改
     */
    private void handleUpdateRelatedWithGroup(Nameable[] nameables) {
        if(nameables==null||nameables.length==0){
            return;
        }
        TableDataConfig config=TableDataConfig.getInstance();
        Map<String,TableData> oldTableDataMap=config.getTableDatas();
        Map<String,TableData> newTableDataMap=new HashMap<>(GroupTableDataTree.TABLE_DATA_SET_INITIAL_SIZE);
        for(Nameable nameable:nameables){
            NameObject no=(NameObject) nameable;
            newTableDataMap.put(no.getName(),(TableData)no.getObject());
        }
        Map<String,TableData> removedNormalTableDataMap=new HashMap<>(GroupTableDataTree.TABLE_DATA_SET_INITIAL_SIZE);
        Map<String,DSGroupTableData> removedGroupTableDataMap=new HashMap<>(GroupTableDataTree.TABLE_DATA_SET_INITIAL_SIZE);
        for(Map.Entry<String,TableData> entry:oldTableDataMap.entrySet()){
            String name=entry.getKey();
            if(!super.getDsNameChangedMap().containsKey(name)&&!newTableDataMap.containsKey(name)){
                TableData tableData=entry.getValue();
                if(tableData instanceof DSGroupTableData){
                    removedGroupTableDataMap.put(name,(DSGroupTableData)tableData);
                }else{
                    removedNormalTableDataMap.put(name,tableData);
                }
            }
        }
        //deal with changing names
        for(Map.Entry<String,String> entry:super.getDsNameChangedMap().entrySet()){
            String newName=entry.getValue();
            String oldName=entry.getKey();
            for(Nameable nameable:nameables){
                NameObject no=(NameObject) nameable;
                TableData tableData=(TableData)no.getObject();
                if(tableData instanceof DSGroupTableData){
                    DSGroupTableData groupData=(DSGroupTableData) tableData;
                    if(groupData.hasChild(oldName)){
                        groupData.changeChildName(oldName,newName);
                    }
                    if(groupData.isChildOf(oldName)){
                        groupData.setParentDSName(newName);
                    }
                }
            }
        }
        //deal with removing normal tabledata
        for(Map.Entry<String,TableData> entry:removedNormalTableDataMap.entrySet()){
            String removedName=entry.getKey();
            for(Nameable nameable:nameables){
                NameObject no=(NameObject) nameable;
                TableData tableData=(TableData)no.getObject();
                if(tableData instanceof DSGroupTableData){
                    DSGroupTableData groupData=(DSGroupTableData) tableData;
                    if(groupData.hasChild(removedName)){
                        groupData.removeChild(removedName);
                    }
                }
            }
        }
        //deal with removing group tabledata
        this.moveChildrenOfRemovedGroupToHigherGroup(removedGroupTableDataMap,newTableDataMap);






        /*for(Map.Entry<String,String> entry:super.getDsNameChangedMap().entrySet()){
            String newName=entry.getValue();
            String oldName=entry.getKey();
            String separator= DSGroupTableData.getSeparator();
            String newNameToReplaceWith=separator+newName+separator;
            String oldNameToMatchWith=separator+oldName+separator;
            for(Nameable nameable:nameables){
                TableData tableData=(TableData) ((NameObject)nameable).getObject();
                if(tableData instanceof DSGroupTableData){
                    DSGroupTableData dsGroupTableData=(DSGroupTableData) tableData;
                    String childNodes=dsGroupTableData.getChildrenDSNames();
                    if(childNodes!=null){
                        childNodes=separator+childNodes+separator;
                        if(childNodes.indexOf(oldNameToMatchWith)>-1){
                            childNodes=childNodes.replace(oldNameToMatchWith,newNameToReplaceWith).replaceAll("^"+separator+"{1}","").replaceAll(separator+"${1}","");
                            dsGroupTableData.setChildrenDSNames(childNodes);
                        }
                    }

                    String parentNode=dsGroupTableData.getParentDSName();
                    if(oldName.equals(parentNode)){
                        parentNode=newName;
                        dsGroupTableData.setParentDSName(parentNode);
                    }
                }
            }
        }*/

    }

    private void moveChildrenOfRemovedGroupToHigherGroup(Map<String,DSGroupTableData> removedGroupDataMap,Map<String,TableData> finalTableDatas){
        while(removedGroupDataMap.size()>0){
            Set<String> removedNames=new HashSet<>();
            for(Map.Entry<String,DSGroupTableData> group:removedGroupDataMap.entrySet()){
                boolean hasAnyChildGroupOfThisNotBeenHandled=false;
                DSGroupTableData groupData=group.getValue();
                String removedName=group.getKey();
                if(groupData.hasChildren()){
                    String children=groupData.getChildrenDSNames();
                    String[] childNames=children.split(DSGroupTableData.getSeparator());
                    for(String childName:childNames){
                        if(removedGroupDataMap.containsKey(childName)){
                            hasAnyChildGroupOfThisNotBeenHandled=true;
                        }
                    }
                    if(hasAnyChildGroupOfThisNotBeenHandled){
                        continue;
                    }

                    if(groupData.hasParent()){
                        String parentName=groupData.getParentDSName();
                        if(!removedGroupDataMap.containsKey(parentName)){
                            //父类没被删除
                            ((DSGroupTableData)finalTableDatas.get(parentName)).changeChildName(removedName,children);
                        }else{
                            removedGroupDataMap.get(parentName).changeChildName(removedName,children);
                        }
                    }

                }
                removedNames.add(removedName);
            }
            for(String name:removedNames){
                removedGroupDataMap.remove(name);
            }
        }


    }

    /*@Override
    public void update(TableDataConfig dataConfig){
        LinkedHashMap<String,TableData> tableDatas = new LinkedHashMap();
        Nameable[] tableDataArray = this.update();
        for (int i = 0; i < tableDataArray.length; i++) {
            NameObject nameObject = (NameObject) tableDataArray[i];
            if(changedDatas.containsKey(nameObject.getName())){
                tableDatas.put(nameObject.getName(),changedDatas.get(nameObject.getName()));
            }else{
                tableDatas.put(nameObject.getName(), (TableData) nameObject.getObject());
            }

        }
        dataConfig.setTableDatas(tableDatas);
        changedDatas.clear();
    }*/


    }
