package com.fr.plugins.dsgroup.fun;


import com.fr.config.holder.Conf;
import com.fr.config.holder.impl.SimConf;
import com.fr.data.AbstractParameterTableData;
import com.fr.design.mainframe.WestRegionContainerPane;
import com.fr.general.data.DataModel;
import com.fr.script.Calculator;
import com.fr.stable.StringUtils;
import com.fr.stable.xml.XMLPrintWriter;
import com.fr.stable.xml.XMLableReader;

/**
 * Created by richie on 2016/10/8.
 */
public class DSGroupTableData extends AbstractParameterTableData {

    private static final String XML_ATTR_TAG = "QueryTableDataAttr";

    private static final String separator="&;";
    private static final String childrenAttrName="children_DS_names";
    private static final String parentAttrName="parent_DS_name";
    private Conf<String> attrParentDSName=new SimConf<>(null);
    private Conf<String> attrChildrenDSNames=new SimConf<>(null);

    public Conf<String> getAttrParentDSName() {
        return attrParentDSName;
    }

    public void setAttrParentDSName(Conf<String> attrParentDSName) {
        this.attrParentDSName = attrParentDSName;
    }

    public Conf<String> getAttrChildrenDSNames() {
        return attrChildrenDSNames;
    }

    public void setAttrChildrenDSNames(Conf<String> attrChildrenDSNames) {
        this.attrChildrenDSNames = attrChildrenDSNames;
    }

    public static String getSeparator() {
        return separator;
    }

    public String getChildrenDSNames() {
        return attrChildrenDSNames.get();
    }

    public boolean hasChildren(){
        return attrChildrenDSNames.get()!=null;
    }

    public boolean hasParent(){
        return attrParentDSName.get()!=null;
    }

    public void setChildrenDSNames(String childrenDSNames) {
        this.attrChildrenDSNames.set(childrenDSNames);
    }

    public String getParentDSName() {
        return attrParentDSName.get();
    }

    public void setParentDSName(String parentDSName) {
        this.attrParentDSName.set(parentDSName);
    }

    public void addChildren(String children){
        if(StringUtils.isEmpty(children)){
            return;
        }
        if(this.hasChildren()){
            this.setChildrenDSNames(this.getChildrenDSNames()+separator+children);
        }else{
            this.setChildrenDSNames(children);
        }
    }

    public void removeParent(){
        this.setParentDSName(null);
    }

    public void removeChild(String childName){
        if(!hasChildren()){
            return;
        }
        String children=this.getChildrenDSNames();
        children=wrapDsName(children);
        childName=wrapDsName(childName);
        children=children.replace(childName,separator).replaceAll("^"+separator+"{1}","").replaceAll(separator+"${1}","");
        if(children.length()==0){
            children=null;
        }
        this.setChildrenDSNames(children);
    }

    public boolean hasChild(String childName){
        if(!hasChildren()){
            return false;
        }else{
            childName=wrapDsName(childName);
            String myChildren=wrapDsName(this.getChildrenDSNames());
            return myChildren.indexOf(childName)>-1;
        }
    }

    public boolean isChildOf(String parentName){
        return parentName.equals(this.getParentDSName());
    }

    public void changeChildName(String oldChildName,String newChildName){
        if(!hasChildren()||StringUtils.isEmpty(newChildName)){
            return;
        }
        String children=this.getChildrenDSNames();
        children=wrapDsName(children);
        oldChildName=wrapDsName(oldChildName);
        newChildName=wrapDsName(newChildName);
        children=children.replace(oldChildName,newChildName).replaceAll("^"+separator+"{1}","").replaceAll(separator+"${1}","");
        this.setChildrenDSNames(children);
    }

    private String wrapDsName(String name){
        return separator+name+separator;
    }

    @Override
    public DataModel createDataModel(Calculator calculator) {
        //return new QueryDataModel(processParameters(calculator));
        return null;
    }

    @Override
    public void readXML(XMLableReader reader) {
        super.readXML(reader);
        if (reader.isChildNode()) {
            String tagName = reader.getTagName();
            if (tagName.equals(XML_ATTR_TAG)) {
                attrParentDSName = new SimConf<>(reader.getAttrAsString(parentAttrName, null));
                attrChildrenDSNames = new SimConf<>(reader.getAttrAsString(childrenAttrName, null));
            }
        }
    }

    @Override
    public void writeXML(XMLPrintWriter writer) {
        super.writeXML(writer);
        writer.startTAG(XML_ATTR_TAG);
        writer.attr(parentAttrName, attrParentDSName.get());
        writer.attr(childrenAttrName, attrChildrenDSNames.get());
        writer.end();
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        DSGroupTableData cloned = (DSGroupTableData)super.clone();
        cloned.attrParentDSName=(SimConf<String>)this.attrParentDSName.clone();
        cloned.attrChildrenDSNames=(SimConf<String>)this.attrChildrenDSNames.clone();
        return cloned;
    }

    public DSGroupTableData newData(){
        try{
            return (DSGroupTableData)this.clone();
        }catch (CloneNotSupportedException e){
            return null;
        }
    }
}
