package com.fr.plugins.dsgroup.rpc;

import com.fr.base.TableDataBean;
import com.fr.file.ConfigChangeProcessor;
import com.fr.file.TableDataConfig;
import com.fr.transaction.Configurations;
import com.fr.transaction.WorkerFacade;

import java.util.List;

/**
 * @author phetran_ye
 * 2022-04-20
 */
public class GroupTableDataOperatorImpl implements GroupTableDataOperator{
    private static final GroupTableDataOperator INSTANCE = new GroupTableDataOperatorImpl();

    public static GroupTableDataOperator getInstance() {
        return INSTANCE;
    }

    @Override
    public boolean saveTableData(List<String> toDeleted, List<TableDataBean> tableDataBeanList) throws Exception {
        return save(tableDataBeanList, new Runnable() {
            @Override
            public void run() {
                removeDeleted(toDeleted);
            }
        });
    }

    private boolean save(List<TableDataBean> tableDataBeanList, Runnable deleteAction) {
        ConfigChangeProcessor.getInstance().applyForTableData(tableDataBeanList);
        return Configurations.modify(new WorkerFacade(TableDataConfig.class) {
            @Override
            public void run() {
                deleteAction.run();
                for (TableDataBean tableDataBean : tableDataBeanList) {
                    TableDataConfig.getInstance().addTableData(tableDataBean.getName(), tableDataBean.getTableData());
                }
            }
        });
    }

    private void removeDeleted(List<String> toDeleted) {
        for (String name : toDeleted) {
            TableDataConfig.getInstance().removeTableData(name);
        }
    }
}
